@@ -146,6 +146,8 @@ def resume_pipeline(pipeline_id):
     :param pipeline_id:
     :return:
     """
+    if not Status.objects.filter(id=pipeline_id).exists():
+        return ActionResult(result=False, message="only started pipeline can be resumed.")
 
     action_result = Status.objects.transit(id=pipeline_id, to_state=states.READY, is_pipeline=True, appoint=True)
     if not action_result.result:
@@ -367,7 +369,9 @@ def get_status_tree(node_id, max_depth=1):
     """
     rel_qs = NodeRelationship.objects.filter(ancestor_id=node_id, distance__lte=max_depth)
     if not rel_qs.exists():
-        raise exceptions.InvalidOperationException("node(%s) does not exist, may have not by executed" % node_id)
+        raise exceptions.InvalidOperationException(
+            "node(%s) does not exist, may have not by executed or expired" % node_id
+        )
     descendants = [rel.descendant_id for rel in rel_qs]
     # remove root node
     descendants.remove(node_id)
