@@ -21,8 +21,10 @@ from kombu import Exchange, Queue, Connection
 
 from bamboo_engine import states
 from bamboo_engine.template import Template
+from bamboo_engine.eri import interfaces
 from bamboo_engine.eri import EngineRuntimeInterface, NodeType, ContextValueType
 
+from pipeline.eri import codec
 from pipeline.eri.imp.plugin_manager import PipelinePluginManagerMixin
 from pipeline.eri.imp.hooks import HooksMixin
 from pipeline.eri.imp.process import ProcessMixin
@@ -57,12 +59,32 @@ class BambooDjangoRuntime(
         "lazy": ContextValueType.COMPUTE.value,
     }
 
+    ERI_SUPPORT_VERSION = 2
+
+    def __init__(self):
+        try:
+            eri_version = interfaces.version()
+        except AttributeError:
+            raise RuntimeError(
+                "bamboo_engine eri do not support version fetch, please make sure bamboo_engine version >= 1.1.6"
+            )
+
+        major_version = int(eri_version.split(".")[0])
+        if major_version > self.ERI_SUPPORT_VERSION:
+            raise RuntimeError(
+                "unsupported bamboo_engine eri version: %s, expect version: <= %s.x.x"
+                % (eri_version, self.ERI_SUPPORT_VERSION)
+            )
+
     def _data_inputs_assemble(self, pipeline_id: str, node_id: str, node_inputs: dict) -> (dict, List[ContextValue]):
         inputs = {}
         context_values = []
         for k, v in node_inputs.items():
             if v["type"] == "lazy":
-                cv_key = "${%s_%s}" % (k, node_id)
+                if k.startswith("${") and k.endswith("}"):
+                    cv_key = "${%s_%s}" % (k[2:-1], node_id)
+                else:
+                    cv_key = "${%s_%s}" % (k, node_id)
                 if len(cv_key) > 128:
                     raise ValueError("var key %s length exceeds 128" % cv_key)
                 context_values.append(
@@ -202,7 +224,7 @@ class BambooDjangoRuntime(
         )
 
     def _prepare(
-        self, pipeline: dict, root_id: str, parent_id: Optional[str] = None
+        self, pipeline: dict, root_id: str, subprocess_context: dict, parent_id: Optional[str] = None
     ) -> (List[Node], List[Data], List[ContextValue], List[ContextOutputs]):
 
         parent_id = parent_id or root_id
@@ -214,15 +236,15 @@ class BambooDjangoRuntime(
 
         node_outputs = {}
         context_var_references = {}
-        final_refrences = {}
+        final_references = {}
 
-        # collect all node outputs and initial refrence
+        # collect all node outputs and initial reference
         for key, input_data in pipeline["data"]["inputs"].items():
             source_act = input_data.get("source_act")
             source_key = input_data.get("source_key")
             if not source_act:
                 context_var_references[key] = Template(input_data["value"]).get_reference()
-                final_refrences[key] = set()
+                final_references[key] = set()
                 context_values.append(
                     ContextValue(
                         pipeline_id=pipeline["id"],
@@ -240,6 +262,18 @@ class BambooDjangoRuntime(
                 else:
                     node_outputs.setdefault(source_act, {})[source_key] = key
 
+        # pre_render_keys in start_event
+        if "pre_render_keys" in pipeline["data"] and pipeline["data"]["pre_render_keys"]:
+            datas.append(
+                Data(
+                    node_id=pipeline["start_event"]["id"],
+                    inputs=codec.data_json_dumps(
+                        {"pre_render_keys": {"need_render": False, "value": pipeline["data"]["pre_render_keys"]}}
+                    ),
+                    outputs={},
+                )
+            )
+
         # process activities
         for act in pipeline["activities"].values():
             if act["type"] == NodeType.ServiceActivity.value:
@@ -250,14 +284,14 @@ class BambooDjangoRuntime(
                 datas.append(
                     Data(
                         node_id=act["id"],
-                        inputs=json.dumps(data_inputs),
+                        inputs=codec.data_json_dumps(data_inputs),
                         outputs=json.dumps(node_outputs.get(act["id"], {})),
                     )
                 )
                 # compute context values
                 for cv in compute_cvs:
                     context_values.append(cv)
-                    final_refrences[cv.key] = set()
+                    final_references[cv.key] = set()
                     context_var_references[cv.key] = Template(cv.value).get_reference()
 
             elif act["type"] == NodeType.SubProcess.value:
@@ -270,22 +304,40 @@ class BambooDjangoRuntime(
                 datas.append(
                     Data(
                         node_id=act["id"],
-                        inputs=json.dumps(data_inputs),
+                        inputs=codec.data_json_dumps(data_inputs),
                         outputs=json.dumps(node_outputs.get(act["id"], {})),
                     )
                 )
                 # compute context values
                 for cv in compute_cvs:
                     context_values.append(cv)
-                    final_refrences[cv.key] = set()
+                    final_references[cv.key] = set()
                     context_var_references[cv.key] = Template(cv.value).get_reference()
 
                 # subprocess output
                 context_outputs.append(
                     ContextOutputs(pipeline_id=act["id"], outputs=json.dumps(act["pipeline"]["data"]["outputs"]))
                 )
+
+                # subprocess preset context
+                for key, value in subprocess_context.items():
+                    serialized, serializer = self._serialize(value)
+                    context_values.append(
+                        ContextValue(
+                            pipeline_id=act["id"],
+                            key=key,
+                            type=self.CONTEXT_VALUE_TYPE_MAP["plain"],
+                            serializer=serializer,
+                            value=serialized,
+                            references="[]",
+                        )
+                    )
+
                 sub_nodes, sub_datas, sub_ctx_values, sub_ctx_outputs = self._prepare(
-                    pipeline=act["pipeline"], root_id=root_id, parent_id=act["id"]
+                    pipeline=act["pipeline"],
+                    root_id=root_id,
+                    subprocess_context=subprocess_context,
+                    parent_id=act["id"],
                 )
 
                 nodes.extend(sub_nodes)
@@ -318,7 +370,7 @@ class BambooDjangoRuntime(
                 self._gen_gateway_node(gateway=gateway, pipeline=pipeline, root_id=root_id, parent_id=parent_id)
             )
 
-        # resolve final refrences (BFS)
+        # resolve final references (BFS)
         # convert a:b, b:c,d -> a:b,c,d b:c,d
         for key, references in context_var_references.items():
             queue = []
@@ -328,17 +380,17 @@ class BambooDjangoRuntime(
                 r = queue.pop()
 
                 # processed
-                if r in final_refrences[key]:
+                if r in final_references[key]:
                     continue
 
-                final_refrences[key].add(r)
+                final_references[key].add(r)
                 if r in context_var_references:
                     queue.extend(context_var_references[r])
 
         for cv in context_values:
             if cv.pipeline_id != parent_id:
                 continue
-            fr = final_refrences.get(cv.key)
+            fr = final_references.get(cv.key)
             cv.references = json.dumps(list(fr)) if fr else "[]"
 
         if parent_id == root_id:
@@ -346,7 +398,9 @@ class BambooDjangoRuntime(
 
         return nodes, datas, context_values, context_outputs
 
-    def prepare_run_pipeline(self, pipeline: dict, root_pipeline_data: dict, **options) -> int:
+    def prepare_run_pipeline(
+        self, pipeline: dict, root_pipeline_data: dict, root_pipeline_context: dict, subprocess_context: dict, **options
+    ) -> int:
         """
         进行 pipeline 执行前的准备工作，并返回 进程 ID，该函数执行完成后即代表
         pipeline 是随时可以通过 execute(process_id, start_event_id) 启动执行的
@@ -358,7 +412,11 @@ class BambooDjangoRuntime(
         :param pipeline: pipeline 描述对象
         :type pipeline: dict
         :param root_pipeline_data 根流程数据
-        :type root_pipeline_data dict
+        :type root_pipeline_data: dict
+        :param root_pipeline_context 根流程上下文
+        :type root_pipeline_context: dict
+        :param subprocess_context 子流程预置流程上下文
+        :type subprocess_context: dict
         :return: 进程 ID
         :rtype: str
         """
@@ -367,14 +425,30 @@ class BambooDjangoRuntime(
         priority = options.get("priority", 100)
         pipeline_id = pipeline["id"]
 
-        nodes, datas, context_values, context_outputs = self._prepare(pipeline=pipeline, root_id=pipeline["id"])
+        nodes, datas, context_values, context_outputs = self._prepare(
+            pipeline=pipeline, root_id=pipeline["id"], subprocess_context=subprocess_context
+        )
         datas.append(
             Data(
                 node_id=pipeline_id,
-                inputs=json.dumps({k: {"need_render": False, "value": v} for k, v in root_pipeline_data.items()}),
+                inputs=codec.data_json_dumps(
+                    {k: {"need_render": False, "value": v} for k, v in root_pipeline_data.items()}
+                ),
                 outputs="{}",
             )
         )
+        for key, value in root_pipeline_context.items():
+            serialized, serializer = self._serialize(value)
+            context_values.append(
+                ContextValue(
+                    pipeline_id=pipeline_id,
+                    key=key,
+                    type=self.CONTEXT_VALUE_TYPE_MAP["plain"],
+                    serializer=serializer,
+                    value=serialized,
+                    references="[]",
+                )
+            )
         batch_size = getattr(settings, "BAMBOO_DJANGO_ERI_PREPARE_BATCH_SIZE", 500)
 
         with transaction.atomic():
